@@ -1,6 +1,9 @@
 """Tasmota light."""
+from __future__ import annotations
+
+import colorsys
 import logging
-from distutils.version import LooseVersion
+from typing import Any, cast
 
 import attr
 
@@ -19,7 +22,6 @@ from .const import (
     CONF_MAC,
     CONF_OPTIONS,
     CONF_RELAY,
-    CONF_SW_VERSION,
     CONF_TUYA,
     LST_COLDWARM,
     LST_NONE,
@@ -39,7 +41,7 @@ from .entity import (
     TasmotaEntity,
     TasmotaEntityConfig,
 )
-from .mqtt import send_commands
+from .mqtt import ReceiveMessage, send_commands
 from .utils import (
     config_get_friendlyname,
     config_get_state_offline,
@@ -83,8 +85,11 @@ _LOGGER = logging.getLogger(__name__)
 class TasmotaLightConfig(TasmotaAvailabilityConfig, TasmotaEntityConfig):
     """Tasmota light configuation."""
 
+    idx: int = attr.ib()
+
     dimmer_cmd: str = attr.ib()
     dimmer_state: str = attr.ib()
+    color_suffix: str = attr.ib()
     command_topic: str = attr.ib()
     control_by_channel: bool = attr.ib()
     fade_fixed_duration: bool = attr.ib()
@@ -94,15 +99,17 @@ class TasmotaLightConfig(TasmotaAvailabilityConfig, TasmotaEntityConfig):
     not_power_linked: bool = attr.ib()
     poll_topic: str = attr.ib()
     result_topic: str = attr.ib()
-    rgbw: bool = attr.ib()
     state_power_off: str = attr.ib()
     state_power_on: str = attr.ib()
     state_topic: str = attr.ib()
     tuya: bool = attr.ib()
 
     @classmethod
-    def from_discovery_message(cls, config, idx, platform):
+    def from_discovery_message(
+        cls, config: dict, idx: int, platform: str
+    ) -> TasmotaLightConfig:
         """Instantiate from discovery message."""
+        color_suffix = ""
         dimmer_cmd = COMMAND_DIMMER
         dimmer_state = COMMAND_DIMMER
         control_by_channel = False  # Use Channel<n> command to control the light
@@ -123,6 +130,7 @@ class TasmotaLightConfig(TasmotaAvailabilityConfig, TasmotaEntityConfig):
             if idx - first_light == 0:
                 dimmer_idx = 1  # Brightness controlled by DIMMER1
                 light_type = LIGHT_TYPE_RGB
+                color_suffix = "="
             if idx - first_light == 1:
                 dimmer_idx = 2  # Brightness controlled by DIMMER2
                 if tasmota_light_sub_type == LST_RGBW:
@@ -137,13 +145,6 @@ class TasmotaLightConfig(TasmotaAvailabilityConfig, TasmotaEntityConfig):
         if config[CONF_OPTIONS][OPTION_REDUCED_CT_RANGE]:
             min_mireds = REDUCED_MIN_MIREDS
             max_mireds = REDUCED_MAX_MIREDS
-        support_rgbw = LooseVersion(config[CONF_SW_VERSION]) >= LooseVersion("9.4.0.4")
-        if support_rgbw and light_type == LIGHT_TYPE_RGBW:
-            rgbw = True
-            dimmer_idx = 4  # Brightness controlled by DIMMER4
-            dimmer_cmd = f"{COMMAND_DIMMER}{dimmer_idx}"
-        else:
-            rgbw = False
 
         return cls(
             endpoint="light",
@@ -158,6 +159,7 @@ class TasmotaLightConfig(TasmotaAvailabilityConfig, TasmotaEntityConfig):
             availability_online=config_get_state_online(config),
             dimmer_cmd=dimmer_cmd,
             dimmer_state=dimmer_state,
+            color_suffix=color_suffix,
             command_topic=get_topic_command(config),
             control_by_channel=control_by_channel,
             fade_fixed_duration=config[CONF_OPTIONS][OPTION_FADE_FIXED_DURATION],
@@ -166,7 +168,6 @@ class TasmotaLightConfig(TasmotaAvailabilityConfig, TasmotaEntityConfig):
             min_mireds=min_mireds,
             not_power_linked=config[CONF_OPTIONS][OPTION_NOT_POWER_LINKED],
             result_topic=get_topic_stat_result(config),
-            rgbw=rgbw,
             state_power_off=config_get_state_power_off(config),
             state_power_on=config_get_state_power_on(config),
             state_topic=get_topic_tele_state(config),
@@ -177,20 +178,25 @@ class TasmotaLightConfig(TasmotaAvailabilityConfig, TasmotaEntityConfig):
 class TasmotaLight(TasmotaAvailability, TasmotaEntity):
     """Representation of a Tasmota light."""
 
-    def __init__(self, **kwds):
+    _cfg: TasmotaLightConfig
+
+    def __init__(self, **kwds: Any):
         """Initialize."""
         self._brightness = None
-        self._color = None
-        self._color_temp = None
-        self._state = None
-        self._sub_state = None
+        self._color: list[float] | None = None
+        self._color_temp: int | None = None
+        self._state: bool | None = None
+        self._sub_state: dict | None = None
         super().__init__(**kwds)
 
-    async def subscribe_topics(self):
+    async def subscribe_topics(self) -> None:
         """Subscribe to topics."""
 
-        def state_message_received(msg):
+        def state_message_received(msg: ReceiveMessage) -> None:
             """Handle new MQTT state messages."""
+            if not self._on_state_callback:
+                return
+
             attributes = {}
             idx = self._cfg.idx
 
@@ -218,13 +224,20 @@ class TasmotaLight(TasmotaAvailability, TasmotaEntity):
                             self._color = color
                             attributes["color"] = color
 
+                    color_hsb = get_value_by_path(msg.payload, ["HSBColor"])
+                    if color_hsb is not None:
+                        color_hsb = color_hsb.split(",", 3)
+                        if len(color_hsb) == 3:
+                            color_hs = [float(color_hsb[0]), float(color_hsb[1])]
+                            attributes["color_hs"] = color_hs
+
                     color_temp = get_value_by_path(msg.payload, [COMMAND_CT])
                     if color_temp is not None:
                         self._color_temp = color_temp
                         attributes["color_temp"] = color_temp
 
                     scheme = get_value_by_path(msg.payload, [COMMAND_SCHEME])
-                    if scheme is not None:
+                    if scheme is not None and self.effect_list:
                         try:
                             attributes["effect"] = self.effect_list[scheme]
                         except IndexError:
@@ -235,7 +248,7 @@ class TasmotaLight(TasmotaAvailability, TasmotaEntity):
                     if white_value is not None:
                         attributes["white_value"] = white_value
 
-            state = get_state_power(msg.payload, idx)
+            state = get_state_power(cast(str, msg.payload), idx)
 
             if state == self._cfg.state_power_on:
                 self._state = True
@@ -264,35 +277,35 @@ class TasmotaLight(TasmotaAvailability, TasmotaEntity):
             topics,
         )
 
-    async def unsubscribe_topics(self):
+    async def unsubscribe_topics(self) -> None:
         """Unsubscribe to all MQTT topics."""
         self._sub_state = await self._mqtt_client.unsubscribe(self._sub_state)
 
     @property
-    def effect_list(self):
+    def effect_list(self) -> list[str] | None:
         """Return effect list."""
         if self._cfg.endpoint == "light":
             return ["None", "Wake up", "Cycle up", "Cycle down", "Random"]
         return None
 
     @property
-    def light_type(self):
+    def light_type(self) -> int:
         """Return light type."""
         if self._cfg.endpoint == "light":
             return self._cfg.light_type
         return LIGHT_TYPE_NONE
 
     @property
-    def min_mireds(self):
+    def min_mireds(self) -> int:
         """Return the coldest color_temp that this light supports."""
         return self._cfg.min_mireds
 
     @property
-    def max_mireds(self):
+    def max_mireds(self) -> int:
         """Return the warmest color_temp that this light supports."""
         return self._cfg.max_mireds
 
-    def set_state(self, state, attributes):
+    def set_state(self, state: bool, attributes: dict[str, Any]) -> None:
         """Turn the light on or off."""
         if self._cfg.endpoint == "relay":
             self._set_state_relay(state)
@@ -300,11 +313,11 @@ class TasmotaLight(TasmotaAvailability, TasmotaEntity):
             self._set_state_light(state, attributes)
 
     @property
-    def supports_transition(self):
+    def supports_transition(self) -> bool:
         """Return if the light supports transitions."""
         return self.light_type != LIGHT_TYPE_NONE and not self._cfg.tuya
 
-    def _set_state_relay(self, state):
+    def _set_state_relay(self, state: bool) -> None:
         """Turn the relay on or off."""
         payload = self._cfg.state_power_on if state else self._cfg.state_power_off
         command = f"{COMMAND_POWER}{self._cfg.idx+1}"
@@ -313,13 +326,15 @@ class TasmotaLight(TasmotaAvailability, TasmotaEntity):
             payload,
         )
 
-    def _set_state_light(self, state, attributes):
+    def _set_state_light(self, state: bool, attributes: dict[str, Any]) -> None:
         idx = self._cfg.idx
 
-        commands = []
+        commands: list[tuple[str, str | float]] = []
         transition = attributes.get("transition", 0)
         do_transition = transition > 0
 
+        argument: str | float
+
         # Set fade
         if self.supports_transition and "transition" in attributes:
             command = COMMAND_FADE
@@ -345,31 +360,23 @@ class TasmotaLight(TasmotaAvailability, TasmotaEntity):
 
         commands.append((command, argument))
 
-        if "color" in attributes and len(attributes["color"]) == 4:
+        if "color" in attributes:
             color = attributes["color"]
-            if self._cfg.rgbw:
-                argument = f"{color[0]},{color[1]},{color[2]},{color[3]}"
-                command = f"{COMMAND_COLOR}2"
-                commands.append((command, argument))
-            elif color[3] == 0:
-                argument = f"{color[0]},{color[1]},{color[2]}"
-                command = f"{COMMAND_COLOR}2"
-                commands.append((command, argument))
-            else:
-                white_value_normalized = color[3] / 255
-                argument = min(round(white_value_normalized * 100), 100)
-                command = COMMAND_WHITE
-                commands.append((command, argument))
-        elif "color" in attributes:
-            color = attributes["color"]
-            argument = f"{color[0]},{color[1]},{color[2]}"
+            argument = f"{color[0]},{color[1]},{color[2]}{self._cfg.color_suffix}"
             command = f"{COMMAND_COLOR}2"
             commands.append((command, argument))
+        if "color_hs" in attributes:
+            argument = round(attributes["color_hs"][0])
+            command = "HsbColor1"  # Hue
+            commands.append((command, argument))
+            argument = round(attributes["color_hs"][1])
+            command = "HsbColor2"  # Saturation
+            commands.append((command, argument))
         if "color_temp" in attributes:
             argument = attributes["color_temp"]
             command = COMMAND_CT
             commands.append((command, argument))
-        if "effect" in attributes:
+        if "effect" in attributes and self.effect_list:
             try:
                 effect = attributes["effect"]
                 argument = self.effect_list.index(effect)
@@ -377,6 +384,10 @@ class TasmotaLight(TasmotaAvailability, TasmotaEntity):
                 commands.append((command, argument))
             except ValueError:
                 _LOGGER.debug("Unknown effect %s", effect)
+        if "white_value" in attributes:
+            argument = attributes["white_value"]
+            command = COMMAND_WHITE
+            commands.append((command, argument))
 
         if self._cfg.not_power_linked and "brightness" in attributes:
             # Always send power
@@ -386,7 +397,7 @@ class TasmotaLight(TasmotaAvailability, TasmotaEntity):
 
         send_commands(self._mqtt_client, self._cfg.command_topic, commands)
 
-    def _calculate_speed(self, state, attributes):
+    def _calculate_speed(self, state: bool, attributes: dict[str, Any]) -> float:
         # Calculate speed:
         # Home Assistant's transition is the transition time in seconds.
         # Tasmota's speed command is the number of half-seconds, scaled for a 100% fade
@@ -407,8 +418,18 @@ class TasmotaLight(TasmotaAvailability, TasmotaEntity):
         # Calculate normalized brightness for all channels
         if self.light_type >= LIGHT_TYPE_COLDWARM:
             if self.light_type >= LIGHT_TYPE_RGB and self._color:
+                if "color" in attributes:
+                    new_color = attributes["color"]
+                elif "color_hs" in attributes:
+                    # Convert hs_color to color
+                    color_hs = attributes["color_hs"]
+                    rgb = colorsys.hsv_to_rgb(color_hs[0] / 360, color_hs[1] / 100, 1)
+                    rgb = (int(rgb[0] * 255), int(rgb[1] * 255), int(rgb[2] * 255))
+                    new_color = rgb
+                else:
+                    new_color = self._color
                 now_color = [x / 255 for x in self._color]
-                new_color = [x / 255 for x in attributes.get("color", self._color)]
+                new_color = [x / 255 for x in new_color]
                 now_channels.extend(now_color)
                 new_channels.extend(new_color)
             if self.light_type >= LIGHT_TYPE_COLDWARM and self._color_temp:
@@ -428,9 +449,10 @@ class TasmotaLight(TasmotaAvailability, TasmotaEntity):
             now_channels = [now_brightness / 100]
 
         # Scale transition to the channel with the largest brightness change
-        delta_ratio = max(
-            map(abs, [x1 - x2 for (x1, x2) in zip(now_channels, new_channels)])
+        abs_changes = map(
+            abs, [x1 - x2 for (x1, x2) in zip(now_channels, new_channels)]
         )
+        delta_ratio = max(abs_changes)  # type:ignore[type-var]
         if delta_ratio == 0:
             speed = 0
         else:
